@@ -574,12 +574,17 @@ void do_commandline(int argc, char *argv[])
 		}
 		else if (strcmp(argv[loop], "-bw") == 0)
 		{
-		 	char what = argv[++loop][0];
+			char *what = argv[++loop];
 
-			if (what == 'a' || what == 'f')
-				bufferwhat = what;
+			if (what)
+			{
+				if (what[0] == 'a' || what[0] == 'f')
+					bufferwhat = what[0];
+				else
+					error_exit("-bw expects either 'a' or 'f' as parameter (got: '%c')\n", what[0]);
+			}
 			else
-				error_exit("-bw expects either 'a' or 'f' as parameter (got: '%c')\n", what);
+				error_exit("-bw expects a parameter ('a' or 'f')\n");
 		}
 		else if (strcmp(argv[loop], "--no-repeat") == 0)
 		{
@@ -615,8 +620,14 @@ void do_commandline(int argc, char *argv[])
 		}
 		else if (strcmp(argv[loop], "-H") == 0)
 		{
-			heartbeat_interval = atof(argv[++loop]);
-			if (heartbeat_interval < 0.0) error_exit("value for %s must be >= 0\n", "-H");
+			++loop;
+			if (argv[loop])
+			{
+				heartbeat_interval = atof(argv[loop]);
+				if (heartbeat_interval < 0.0) error_exit("value for %s must be >= 0\n", "-H");
+			}
+			else
+				error_exit("-H requires a parameter\n");
 		}
 		else if (strcasecmp(argv[loop], "-a") == 0)
 		{
@@ -650,19 +661,31 @@ void do_commandline(int argc, char *argv[])
 		}
 		else if (strcmp(argv[loop], "-t") == 0)
 		{
-			win_title = mystrdup(argv[++loop], "-t");
+			++loop;
+			if (!argv[loop])
+				error_exit("-t requires a parameter\n");
+			win_title = mystrdup(argv[loop], "-t");
 		}
 		else if (strcmp(argv[loop], "-x") == 0)
 		{
-			set_title = mystrdup(argv[++loop], "-x");
+			++loop;
+			if (!argv[loop])
+				error_exit("-x requires a parameter\n");
+			set_title = mystrdup(argv[loop], "-x");
 		}
 		else if (strcmp(argv[loop], "-p") == 0)
 		{
-			line_wrap = argv[++loop][0];
-			if (line_wrap == 'o')
-				line_wrap_offset = get_value_arg("-p", argv[++loop], VAL_ZERO_POSITIVE);
-			else if (line_wrap != 'a' && line_wrap != 'l' && line_wrap != 'r' && toupper(line_wrap) != 'S')
-				error_exit("Invalid mode fore -p\n");
+			char *lw = argv[++loop];
+			if (lw)
+			{
+				line_wrap = lw[0];
+				if (line_wrap == 'o')
+					line_wrap_offset = get_value_arg("-p", argv[++loop], VAL_ZERO_POSITIVE);
+				else if (line_wrap != 'a' && line_wrap != 'l' && line_wrap != 'r' && toupper(line_wrap) != 'S')
+					error_exit("Invalid mode fore -p\n");
+			}
+			else
+				error_exit("-p requires a parameter\n");
 		}
 		else if (strcmp(argv[loop], "--retry") == 0)
 		{
@@ -780,15 +803,20 @@ void do_commandline(int argc, char *argv[])
 
 			if (strcasecmp(argv[loop], "-iw") == 0)
 			{
-				if ((argc - loop) > 2 && isdigit(argv[loop+2][0]))
+				if (argv[loop + 2])
 				{
-					is_giw = 1;
-					check_interval = atoi(argv[loop + 2]);
+					if ((argc - loop) > 2 && isdigit(argv[loop+2][0]))
+					{
+						is_giw = 1;
+						check_interval = atoi(argv[loop + 2]);
+					}
+					else
+					{
+						check_interval = 5;
+					}
 				}
 				else
-				{
-					check_interval = 5;
-				}
+					error_exit("-iw requires 2 parameters\n");
 			}
 
 			if (strcmp(argv[loop], "-L") == 0 ||
@@ -1013,14 +1041,19 @@ void do_commandline(int argc, char *argv[])
 		{
 			char *type = argv[++loop];
 
-			if (strcasecmp(type, "atime") == 0)
-				new_only = TT_ATIME;
-			else if (strcasecmp(type, "mtime") == 0)
-				new_only = TT_MTIME;
-			else if (strcasecmp(type, "ctime") == 0)
-				new_only = TT_CTIME;
+			if (type)
+			{
+				if (strcasecmp(type, "atime") == 0)
+					new_only = TT_ATIME;
+				else if (strcasecmp(type, "mtime") == 0)
+					new_only = TT_MTIME;
+				else if (strcasecmp(type, "ctime") == 0)
+					new_only = TT_CTIME;
+				else
+					error_exit("--new-only requires either atime, mtime or ctime as parameter, not '%s'\n", type);
+			}
 			else
-				error_exit("--new-only requires either atime, mtime or ctime as parameter, not '%s'\n", type);
+				error_exit("--new-only requires a parameter\n");
 		}
 		else if (strcasecmp(argv[loop], "-q") == 0)
 		{
