@@ -4,6 +4,7 @@
 #include <regex.h>
 #include <ctype.h>
 #include "mt.h"
+#include "mem.h"
 #include "help.h"
 #include "term.h"
 #include "utils.h"
@@ -28,21 +29,29 @@ char *help_main_menu[] = {
 		"       window)",
 		" ^s^     swap the location of 2 windows",
 		" ^e^     enter/edit regular expressions",
+		" ^f^     enter/edit in line filter regexps",
 		" ^c^     set/change colors (only works if your",
 		"       terminal supports colors)",
+		" ^C^     edit the r/g/b components of a color",
 		" ^b^     scrollback in the buffer of a window",
 		" ^v^     toggle vertical split",
 		" ^0^...^9^ set a mark in a window so that you can",
 		"       easily see what has changed since the last",
 		"       time you looked",
+		" ^Y^     send a signal to a window (for commands)",
+		" ^y^     set linewrap",
 		" ^o^     wipe (clear) a window",
 		" ^O^     wipe all windows",
+		" ^T^     truncate a logfile",
+		" ^g^     make screendump",
 		" ^r^     redraw screen",
 		" ^l^     list keybindings",
 		" ^i^     info",
 		" ^t^     statistics",
 		" ^j^     set window sizes",
-		" ^z^     hide a window temporarily",
+		" ^z^     hide/unhide a window",
+		" ^u^     hide all windows but the one selected",
+		" ^U^     unhide all windows",
 		" ^w^     write a script which starts multitail",
 		"       with the current settings (the files that",
 		"       are monitored, regular expressions, etc.)",
@@ -445,21 +454,27 @@ char *help_write_script[] = {
 	};
 
 char *help_statistics[] = {
+		"In this menu one can select the window or sub-",
+		"window to display the statistics off.",
+		" ^r^     reset ALL counters for ALL windows",
+		" ^ctrl+g^     exit window",
+		NULL,
+	};
+
+char *help_statistics_popup[] = {
 		"Apart from how long MultiTail has been running, it",
 		"also shows a couple of statistics for each window.",
 		"It also shows some statistics on the interval",
 		"between each show line in each window.",
 		" *average*             average interval",
 		" *standard deviation*  speaks for itself",
-		" *tendency*            wether the average tends to",
-		"                     become larger or smaller",
 		"",
-		" ^r^     resets the counters",
-		" ^ctrl+g^     abort",
+		" ^r^     reset the counters for this window",
+		" ^ctrl+g^     exit window",
 		NULL
 	};
 
-char *help_set_mark[] = {
+char *help_set_buffering[] = {
 		"In this window you can control the buffering for",
 		"each window. The first question lets you select",
 		"on what window you want to set the buffering",
@@ -470,7 +485,7 @@ char *help_set_mark[] = {
 		NULL
 	};
 
-char *help_set_mark_store_what[] = {
+char *help_set_buffering_store_what[] = {
 		"Here you set what should be buffered.",
 		" ^a^     buffer all lines that are monitored",
 		" ^m^     only buffer the lines that match the",
@@ -487,7 +502,7 @@ char *help_enter_number_of_lines_to_store[] = {
 		NULL
 	};
 
-char *help_set_mark_select_window[] = {
+char *help_set_buffering_select_window[] = {
 		"Here you select the window on which you want to",
 		"set the buffer limits.",
 		"Press ^ctrl+g^ to abort.",
@@ -532,6 +547,9 @@ char *help_set_window_height[] = {
 		"switch.",
 		"One can only set the window height if there are at",
 		"least 2 windows on the screen.",
+		"When you enter '0' MultiTail will automatically",
+		"set the height. The height must be either 0 or",
+		"more then one.",
 		"Press enter on an empty line to abort.",
 		NULL
 	};
@@ -563,6 +581,9 @@ char *help_scrollback_help[] = {
 		" ^c^     set colors",
 		" ^f^/^/^   search for a string in the buffer",
 		" ^n^     find the next occurence",
+		" ^y^     toggle linewrap. if linewrap is disabled,",
+		"       one can scroll left/right with the cursor-",
+		"       keys.",
 		" ^s^     save the buffer to a file",
 		"Press ^ctrl+g^ to abort.",
 		NULL
@@ -617,22 +638,13 @@ char *help_ask_colors_select_color[] = {
 	};
 
 char *help_set_vertical_split_n_win[] = {
-		/*2345678901234567890123456789012345678901234567890 */
 		"Enter the number of columns to split the terminal",
 		"in. If you press 'j' in the menu, you can also",
 		"select the number of windows per column.",
 		NULL
 	};
 
-char *help_manage_cols[] = {
-		/*2345678901234567890123456789012345678901234567890 */
-		/* FIXME: */
-		"fixme",
-		NULL
-	};
-
 char *help_wipe_window[] = {
-		/*2345678901234567890123456789012345678901234567890 */
 		"Press the number of the window to wipe. The number",
 		"can be found in the status-line of the windows.",
 		"Press 'r' to redraw the windows so that you can",
@@ -640,6 +652,231 @@ char *help_wipe_window[] = {
 		NULL
 	};
 
+char *help_select_colorschemes[] = {
+		"Move the cursor with the cursor keys to the a",
+		"colorscheme. Then press space to select and again",
+		"to unselect. Press enter to submit or ^g to abort.",
+		NULL
+	};
+
+char *help_manage_cols[] = {
+		"Press ^a^ to add a column, ^d^ to delete one and press",
+		"^e^ to set the number of windows in a column.",
+		NULL
+	};
+
+char *help_set_linewrap[] = {
+		"In this menu one can set how the wrap the text in",
+		"windows.",
+		" ^l^  start at the left and cut of at the right",
+		" ^a^  try to display everything",
+		" ^r^  cut off at the left",
+		" ^s^  cut right before the syslog process name",
+		" ^S^  cut right after the syslog process name",
+		" ^o^  enter a position where to cut",
+		" Press ctrl+g to abort.",
+		NULL
+	};
+
+char *help_enter_stripper_select_window[] = {
+		"Select window to apply the stripping to.",
+		NULL
+	};
+
+char *help_enter_stripper_select_subwindow[] = {
+		"Select the sub-window to apply the stripping to.",
+		NULL
+	};
+
+char *help_redirect_failed[] = {
+		"Sending the data which was retrieved from a tail-",
+		"process to some other process or file failed.",
+		"Maybe the disk is full or the process died?",
+		NULL
+	};
+
+char *help_stripper_type[] = {
+		" ^e^   filter by using a regular expression",
+		" ^r^   filter by specifying the character-range",
+		" ^c^   specify a column to strip - you'll be asked",
+		"     to enter a delimiter as well",
+		NULL
+	};
+
+char *help_stripper_start_offset[] = {
+		"Please enter the offset from where to strip the",
+		"string. The offset starts at 0.",
+		NULL
+	};
+
+char *help_stripper_end_offset[] = {
+		"Please enter the offset upto where the string will",
+		"be filtered. Upto, not including!",
+		NULL
+	};
+
+char *help_stripper_delimiter[] = {
+		"When selecting stripping by column-number you need",
+		"to enter the delimiter for each column. Usually",
+		"you would enter a space here.",
+		NULL
+	};
+
+char *help_stripper_col_nr[] = {
+		"Enter the number of the column here. The first one",
+		"is at offset 0.",
+		NULL
+	};
+
+char *help_error_write_script_create_file[] = {
+		"There was an error writing the scriptfile. Maybe",
+		"the filesystem is full or the media is damaged?",
+		NULL
+	};
+
+char *help_column_width[] = {
+		"Enter the number of characters that will fit in a",
+		"line for this column.",
+		NULL
+	};
+
+char *help_n_win_per_col[] = {
+		"Enter the number of windows that will fit in this",
+		"column (vertically).",
+		NULL
+	};
+
+char *help_send_signal_select_window[] = {
+		"Select the window to send the signal to. Please",
+		"note that sending a signal may cause the process",
+		"(for example the tail-process when monitoring a",
+		"logfile) to exit and thereby closing the window.",
+		NULL
+	};
+
+char *help_send_signal_window_send_to_all_subwin[] = {
+		"Would you like to send the signal to all sub-",
+		"windows?",
+		NULL
+	};
+
+char *help_send_signal_select_subwindow[] = {
+		"Select the subwindow to send the signal to.",
+		NULL
+	};
+
+char *help_send_signal_failed[] = {
+		"Sending the signal to the process failed. This",
+		"should not happen.",
+		NULL
+	};
+
+char *help_select_signal[] = {
+		"Select the signal to send. Please note that some",
+		"signals may cause the process to exit and closing",
+		"the window",
+		NULL
+	};
+
+char *help_screendump_select_file[] = {
+		"Select the file to write the screendump to.",
+		"Existing files will be overwritten.",
+		NULL
+	};
+
+char *help_screendump_select_win[] = {
+		"Select the window to dump to a screendump-file.",
+		"The output file will be plain ascii.",
+		NULL
+	};
+
+char *help_truncate_file_select_window[] = {
+		"Select window. By selecting a window you select",
+		"the file to truncate.",
+		NULL
+	};
+
+char *help_truncate_file_select_subwindow[] = {
+		"As the window you selected contains several sub-",
+		"windows, you need to select now the subwindow to",
+		"truncate.",
+		NULL
+	};
+
+char *help_truncate_areyousure[] = {
+		"Are you really sure you want to truncate this",
+		"file? This cannot be undone!",
+		NULL
+	};
+
+char *help_truncate_failed[] = {
+		"There was an error truncating the file. Please",
+		"check the permissions on the file.",
+		NULL
+	};
+
+char *help_truncate_only_logfiles[] = {
+		"One can only truncate logfiles and not processes.",
+		NULL
+	};
+
+char *help_select_color_and_attributes[] = {
+		"Select the foreground and background color to use.",
+		"You can select one foreground and one background-",
+		"color but you can select multiple attributes.",
+		"Press the space-bar to select or un-select.",
+		NULL
+	};
+
+char *help_cannot_change_color[] = {
+		"Your terminal doesn't support chaning the red,",
+		"green and blue values of a color.",
+		NULL
+	};
+
+char *help_edit_color_edit[] = {
+		"Depending on the range you selected (by pressing",
+		"the cursor up and down key), you can enter a value",
+		"for either red, green or blue in the range of",
+		"0-1000, 0-255 or 00-ff (hex value).",
+		NULL
+	};
+
+char *help_edit_color[] = {
+		"In this menu one can edit the values for the red,",
+		"the green and the blue component of a color.",
+		"Move the '>' with the up and down cursorkeys in",
+		"front of the range in which you want to enter the",
+		"value. The uppermost range being 0 upto 1000, the",
+		"middle 0-255 and the lowest also 0-255 but entered",
+		"in hexadecimal values.",
+		" ^r^   edit the red component",
+		" ^g^   edit the green component",
+		" ^b^   edit the blue component",
+		" ^c^   give the color a name/description",
+		NULL
+	};
+
+char *help_edit_color_change_name[] = {
+		"One can give this specific red, green and blue",
+		"values a name. Like roses-red or so. This name can",
+		"then be used in a colorscheme.",
+		NULL
+	};
+
+char *help_hide_but_window[] = {
+		/*2345678901234567890123456789012345678901234567890 */
+		"FIXME", /* FIXME */
+		NULL
+	};
+
+#if 0
+char *help_bla[] = {
+		/*2345678901234567890123456789012345678901234567890 */
+		"FIXME", /* FIXME */
+		NULL
+	};
+#endif
 
 help_t	help[] =
 {
@@ -659,8 +896,11 @@ help_t	help[] =
 	{ HELP_ADD_FILE_DISPLAY_DIFF,		help_add_file_display_diff },
 	{ HELP_MERGE_ANOTHER_WINDOW,		help_merge_another_window },
 	{ HELP_ENTER_REGEXP_SELECT_WINDOW,	help_enter_regexp_select_window },
+	{ HELP_ENTER_STRIPPER_SELECT_WINDOW,	help_enter_stripper_select_window },
 	{ HELP_ENTER_REGEXP_SELECT_SUBWINDOW,	help_enter_regexp_select_subwindow },
+	{ HELP_ENTER_STRIPPER_SELECT_SUBWINDOW,	help_enter_stripper_select_subwindow },
 	{ HELP_REGEXP_MENU,			help_regexp_menu },
+
 	{ HELP_ENTER_REGEXP,			help_enter_regexp },
 	{ HELP_NEGATE_REGEXP,			help_negate_regexp },
 	{ HELP_REGEXP_USAGE,			help_regexp_usage },
@@ -677,10 +917,10 @@ help_t	help[] =
 	{ HELP_WINDOW_CLOSED,			help_window_closed },
 	{ HELP_WRITE_SCRIPT,			help_write_script },
 	{ HELP_STATISTICS,			help_statistics },
-	{ HELP_SET_MARK,			help_set_mark },
-	{ HELP_SET_MARK_STORE_WHAT,		help_set_mark_store_what },
+	{ HELP_SET_BUFFERING,			help_set_buffering },
+	{ HELP_SET_BUFFERING_STORE_WHAT,	help_set_buffering_store_what },
 	{ HELP_ENTER_NUMBER_OF_LINES_TO_STORE,	help_enter_number_of_lines_to_store },
-	{ HELP_SET_MARK_SELECT_WINDOW,	 	help_set_mark_select_window },
+	{ HELP_SET_BUFFERING_SELECT_WINDOW, 	help_set_buffering_select_window },
 	{ HELP_HIDE_WINDOW,			help_hide_window },
 	{ HELP_SET_WINDOWSIZES,			help_set_windowsizes },
 	{ HELP_SET_WINDOW_WIDTH,		help_set_window_width },
@@ -693,15 +933,40 @@ help_t	help[] =
 	{ HELP_SCROLLBACK_SAVEFILE_ENTER_FILENAME,	help_scrollback_savefile_enter_filename },
 	{ HELP_SCROLLBACK_SELECT_WINDOW,	help_scrollback_select_window },
 	{ HELP_SCROLLBACK_EDIT_SEARCH_STRING,	help_scrollback_edit_search_string },
-#if 0
-	{ HELP_SCROLLBACK_DELETE_MARK_NO_MARK,	help_scrollback_delete_mark_no_mark },
-	{ HELP_SCROLLBACK_DELETE_MARK_SELECT_WINDOW,	help_scrollback_delete_mark_select_window },
-#endif
 	{ HELP_PAUSE_A_WINDOW,			help_pause_a_window },
 	{ HELP_ASK_COLORS_SELECT_COLOR,		help_ask_colors_select_color },
 	{ HELP_SET_VERTICAL_SPLIT_N_WIN, 	help_set_vertical_split_n_win },
 	{ HELP_MANAGE_COLS,			help_manage_cols },
 	{ HELP_WIPE_WINDOW,			help_wipe_window },
+	{ HELP_SELECT_COLORSCHEMES,		help_select_colorschemes },
+	{ HELP_SET_LINEWRAP,			help_set_linewrap },
+	{ HELP_REDIRECT_FAILED,			help_redirect_failed },
+	{ HELP_STRIPPER_TYPE,			help_stripper_type },
+	{ HELP_STRIPPER_START_OFFSET,		help_stripper_start_offset },
+	{ HELP_STRIPPER_END_OFFSET,		help_stripper_end_offset },
+	{ HELP_STRIPPER_DELIMITER,		help_stripper_delimiter },
+	{ HELP_STRIPPER_COL_NR,			help_stripper_col_nr },
+	{ HELP_ERROR_WRITE_SCRIPT_CREATE_FILE,	help_error_write_script_create_file },
+	{ HELP_COLUMN_WIDTH,			help_column_width },
+	{ HELP_N_WIN_PER_COL,			help_n_win_per_col },
+	{ HELP_SEND_SIGNAL_SELECT_WINDOW,	help_send_signal_select_window },
+	{ HELP_SEND_SIGNAL_WINDOW_SEND_TO_ALL_SUBWIN, help_send_signal_window_send_to_all_subwin },
+	{ HELP_SEND_SIGNAL_SELECT_SUBWINDOW,	help_send_signal_select_subwindow },
+	{ HELP_SEND_SIGNAL_FAILED,		help_send_signal_failed },
+	{ HELP_SELECT_SIGNAL,			help_select_signal },
+	{ HELP_SCREENDUMP_SELECT_FILE,		help_screendump_select_file },
+	{ HELP_SCREENDUMP_SELECT_WIN,		help_screendump_select_win },
+	{ HELP_TRUNCATE_FILE_SELECT_WINDOW,	help_truncate_file_select_window },
+	{ HELP_TRUNCATE_FILE_SELECT_SUBWINDOW,	help_truncate_file_select_subwindow },
+	{ HELP_TRUNCATE_AREYOUSURE,		help_truncate_areyousure },
+	{ HELP_TRUNCATE_FAILED,			help_truncate_failed },
+	{ HELP_TRUNCATE_ONLY_LOGFILES,		help_truncate_only_logfiles },
+	{ HELP_SELECT_COLOR_AND_ATTRIBUTES,	help_select_color_and_attributes },
+	{ HELP_CANNOT_EDIT_COLOR,               help_cannot_change_color },
+	{ HELP_EDIT_COLOR_EDIT,			help_edit_color_edit },
+	{ HELP_EDIT_COLOR,			help_edit_color },
+	{ HELP_EDIT_COLOR_CHANGE_NAME,		help_edit_color_change_name },
+	{ HELP_HIDE_BUT_WINDOW,			help_hide_but_window },
 	{ -1, NULL }
 };
 
@@ -767,6 +1032,18 @@ void show_help(int what_help)
 			{
 				pos++;
 			}
+			else if (c == KEY_PPAGE && pos >= 15)
+			{
+				pos -= 15;
+			}
+			else if ((c == KEY_NPAGE || c == ' ') && pos < (line_cnt - 1))
+			{
+				pos += 15;
+			}
+			else if (c == KEY_HOME && pos > 0)
+			{
+				pos = 0;
+			}
 			else if (tolower(c) == 'q' || c == 7)
 			{
 				break;
