@@ -303,6 +303,7 @@ void signal_handler(int sig)
 
 	switch(sig)
 	{
+		case SIGHUP:
 		case SIGTERM:
 			do_exit();
 			break;
@@ -2211,7 +2212,8 @@ int main(int argc, char *argv[])
 
 	time(&mt_started);
 
-	if (SIG_ERR == signal(SIGTERM, signal_handler)) error_exit("main: setting of signal failed\n");
+	if (SIG_ERR == signal(SIGTERM, signal_handler)) error_exit("main: setting of handler for TERM signal failed\n");
+	if (SIG_ERR == signal(SIGHUP,  signal_handler)) error_exit("main: setting of handler for HUP signal failed\n");
 
 	mail_spool_file = getenv("MAIL");
 	if (dummy) check_for_mail = atoi(dummy);
